///////////////////////////////////////////////////////////////////
//
// Project RENOIR
// Release Notes Instant Reporter
//
// Reference application presented in
// Clean Architecture with .NET (MS Press) 2024
// Author: Dino Esposito
// 
//

using System.ComponentModel.DataAnnotations;
using Youbiquitous.Renoir.DomainModel.Documents;

namespace Youbiquitous.Renoir.DomainModel;

/// <summary>
/// PRODUCT entity (properties)
/// </summary>
public partial class Product : BaseEntity
{
    /// <summary>
    /// Ctor, mostly needed for EF and to save us an entire layer of DTOs
    /// </summary>
    public Product()
    {
    }

    public Product(string name, string version = "")
    {
        Name = name;
        Version = version;
    }

    public Product(long id, string name, string version = "")
    {
        ProductId = id;
        Name = name;
        Version = version;
    }
    
    /// <summary>
    /// Unique ID (autogenerated)
    /// </summary>
    public long ProductId { get; set; }

    /// <summary>
    /// Product name
    /// </summary>
    [MaxLength(50)]
    public string Name { get; set; }

    /// <summary>
    /// Version number
    /// </summary>
    [MaxLength(30)]
    public string Version { get; set; }

    /// <summary>
    /// Users active on the product
    /// </summary>
    public IEnumerable<UserProductBinding> Users { get; set; }

    /// <summary>
    /// Pending release notes for the product
    /// </summary>
    public IEnumerable<ReleaseNote> ReleaseNotes { get; set; }

    /// <summary>
    /// Pending roadmaps for the product
    /// </summary>
    public IEnumerable<Roadmap> Roadmaps { get; set; }
}
